/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIESREGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2012, 2016 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_main.c
* Version      : Code Generator for RL78/G10 V1.04.04.01 [11 Nov 2016]
* Device(s)    : R5F10Y47
* Tool-Chain   : CCRL
* Description  : This file implements main function.
* Creation Date: 2017/04/20
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_cgc.h"
#include "r_cg_tau.h"
#include "r_cg_it.h"
#include "r_cg_sau.h"
/* Start user code for include. Do not edit comment generated here */
#include "r_iic_lib.h"
/* End user code. Do not edit comment generated here */
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
#pragma interrupt r_it_interrupt(vect=INTIT)
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
#define     MAX_DATA    ( 64 )                  /* max data number for one time */
#define     RETRY       ( 10 )                  /* error retry times            */
#define     BMP180      ( 0xEE )                /* slave address of BMP180      */
#define     CTRL_MEAS   ( 0xF4 )                /* measurement control register */
#define     C_TEMP      ( 0x2E )                /* start temperature measurement*/
#define     A_TEMP      ( 0xF6 )                /* address of temperature       */
#define     A_AC5       ( 0xB2 )                /* address of AC5 data          */
#define     A_AC6       ( 0xB4 )                /* address of AC6 data          */
#define     A_MC        ( 0xBC )                /* address of MC data           */
#define     A_MD        ( 0xBE )                /* address of MD data           */

volatile uint16_t   g_AC5;                      /* Calibration parameter AC5    */
volatile uint16_t   g_AC6;                      /* Calibration parameter AC6    */
volatile uint16_t    g_MC;                      /* Calibration parameter MC     */
volatile uint16_t    g_MD;                      /* Calibration parameter MD     */
volatile uint16_t   g_UT;                       /* temperature A/D data UT      */

volatile uint8_t    g_write_data[MAX_DATA+1];   /* transmit data buffer         */
volatile uint8_t    g_tx_buf[16];               /* transmit data buffer         */
volatile uint8_t    g_rx_buf[16];               /* received data buffer         */


volatile uint8_t    g_005s_status;              /* 0.05s interrupt flag         */
volatile uint8_t    g_005s_count;               /* 0.05s counter                */

uint8_t g_IIC_put_data( uint8_t s_addr, uint8_t r_addr, volatile uint8_t * const buffer, uint8_t tx_num );
uint8_t g_IIC_get_data( uint8_t s_addr, uint8_t r_addr, volatile uint8_t * const buffer, uint8_t rx_num );
uint8_t g_IIC_get_wdata( uint8_t s_addr, uint8_t r_addr, volatile uint16_t * const buffer );
uint8_t g_IIC_get_temp( volatile uint16_t * const buffer);

void wait_5ms( uint8_t );
void wait_250ms(void);
void g_wait_SW(void);


/* End user code. Do not edit comment generated here */

static void R_MAIN_UserInit(void);
/***********************************************************************************************************************
* Function Name: main
* Description  : This function implements main function.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void main(void)
{
    R_MAIN_UserInit();
    /* Start user code. Do not edit comment generated here */
  {

    uint8_t status;

/* readr EEPROM */
    status = g_IIC_get_wdata( BMP180, A_AC5, &g_AC5 );
    if ( 0x00 == status )
    {
        status = g_IIC_get_wdata( BMP180, A_AC6, &g_AC6 );
        status = g_IIC_get_wdata( BMP180, A_MC, &g_MC );
        status = g_IIC_get_wdata( BMP180, A_MD, &g_MD );

/* start temperature measurement    */
        status = g_IIC_get_temp( &g_UT );
    }

  }
    /* End user code. Do not edit comment generated here */
}
/***********************************************************************************************************************
* Function Name: R_MAIN_UserInit
* Description  : This function adds user code before implementing main function.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void R_MAIN_UserInit(void)
{
    /* Start user code. Do not edit comment generated here */

/* use TM03 for delay count mode */

    IIC_TM03_init();                    /* start TM03               */
    R_IICA0_Create();

/* start IT                       */

    g_005s_status = TRUTH;
    g_005s_count = 0x00;
    ITMCH |= 0x80;                     /* start IT                 */
    ITIF = FALSE;                       /* clear interrupt flag	    */
    ITMK = INT_ENABLE;                  /* enable IT interrupt	    */

    EI();

    /* End user code. Do not edit comment generated here */
}

/* Start user code for adding. Do not edit comment generated here */

/***********************************************************************************************************************

    IIC bus device control routines based on r_IIC_LIB.c

***********************************************************************************************************************/

/*******************************************************************************
* Function Name: g_IIC_get_temp
* Description  : This function get temperature from BMP180
* Arguments    : store pointer
* Return Value : 00:successs
*                not 00:error
********************************************************************************/
uint8_t g_IIC_get_temp( volatile uint16_t * const buffer)
{

    uint8_t status;
    volatile uint16_t * ptr;

    ptr = buffer;
    g_tx_buf[0] = C_TEMP;
    status = g_IIC_put_data( BMP180, CTRL_MEAS, &g_tx_buf[0], 1 );
    if ( 0x00 == status )
    {
        wait_5ms(2);
        status = g_IIC_get_wdata( BMP180, A_TEMP, ptr );
    }

    return (status);
}

/*******************************************************************************
* Function Name: g_IIC_put_data
* Description  : This function transmit data to specified slave
* Arguments    : slave address, register address, data buffer address, data numer
* Return Value : 00:successs
*                not 00:error
********************************************************************************/
uint8_t g_IIC_put_data( uint8_t s_addr, uint8_t r_addr, volatile uint8_t * const buffer, uint8_t tx_num )
{
    uint8_t cnt;
    uint8_t cnt2;
    volatile uint8_t * ptr;
    uint8_t status;
    uint8_t retry_count;

    cnt = tx_num;                       /* set transmit data number     */
    g_write_data[0] = r_addr;           /* set register address         */
    ptr = buffer;

    for ( cnt2 = 0 ; cnt2 < cnt ; cnt2++ )
    {

/*--------------------------------------
    copy transmit data to transmit buffer
--------------------------------------*/

        g_write_data[cnt2+1] = *ptr;
        ptr++;
    }

/*--------------------------------------
    start transmission 
--------------------------------------*/
    for ( retry_count = RETRY ; retry_count > 0 ; )
    {

        status = R_IIC_Master_Send(s_addr, (uint8_t *__near)g_write_data, cnt+1);

        if( IIC_SUCCESS == status )     /* check start successfully     */
        {

/*--------------------------------------
    transmit start and 
    wait for transmission end
--------------------------------------*/

            do
            {                           /* wait for transmission end    */

                status = R_IIC_check_comstate(); /* get IIC status      */

            }while( IIC_USING == ( status & IIC_STS_MASK ) );
        
        }
        else
        {
            /* do nothing */
        }
/*--------------------------------------
    transmission end
--------------------------------------*/
        if( NO_SLAVE == status )
        {                               /* no ACK for slave address     */
            R_IIC_StopCondition();      /* issue stop condition         */
            wait_5us();                 /* wait for retry timing        */
            retry_count--;
        }
        else
        {
            retry_count = 0;
        }
    }
/*
    release IIC  bus
*/
    if( NO_SLAVE != status )
    {
        R_IIC_StopCondition();          /* issue stop condition         */
    }
    else
    {
            /* do nothing */
    }

    return (status);

}
/*******************************************************************************
* Function Name: g_IIC_get_wdata
* Description  : This function receive 16bit data from specified slave
* Arguments    : slave address, register address, data buffer address, data numer
* Return Value : 00:successs
*                not 00:error
********************************************************************************/
uint8_t g_IIC_get_wdata( uint8_t s_addr, uint8_t r_addr, volatile uint16_t * const buffer )
{
    uint8_t status;
    uint8_t buff[2];
    uint16_t work;

    status = g_IIC_get_data( s_addr, r_addr, &buff[0], 0x02 );
    if ( 0x00 == status )
    {
        work = (( buff[0] << 8 ) | buff[1] );
        *buffer = work;
    }
    return (status);
}
/*******************************************************************************
* Function Name: g_IIC_get_data
* Description  : This function receive data from specified slave
* Arguments    : slave address, register address, data buffer address, data numer
* Return Value : 00:successs
*                not 00:error
********************************************************************************/
uint8_t g_IIC_get_data( uint8_t s_addr, uint8_t r_addr, volatile uint8_t * const buffer, uint8_t rx_num )
{

    uint8_t status;
    uint8_t retry_count;

    g_write_data[0] = r_addr;           /* set register address         */

    for ( retry_count = RETRY ; retry_count > 0 ; )
    {
/*--------------------------------------
    transmit register address
--------------------------------------*/

        status = R_IIC_Master_Send(s_addr, (uint8_t *__near)g_write_data, 1);

        if( IIC_SUCCESS == status )         /* check start successfully     */
        {

/*--------------------------------------
    wait for transmission end
--------------------------------------*/

            do
            {

                status = R_IIC_check_comstate(); /* get IIC status  */

            }while( IIC_USING == ( status & IIC_STS_MASK ) );

        }
        else
        {
                /* do nothing */
        }

/*--------------------------------------
   transmission end and check result
--------------------------------------*/

        if( NO_SLAVE == status )
        {                               /* no ACK for slave address     */
            R_IIC_StopCondition();      /* issue stop condition         */
            wait_5us();                 /* wait for retry timing        */
            retry_count--;
        }
        else
        {
            retry_count = 0;
        }
    }

/*--------------------------------------
   transmission end
--------------------------------------*/

    if ( IIC_SUCCESS == status )        /* check result         */
    {

/*--------------------------------------
   start IIC receiving
--------------------------------------*/

        status = R_IIC_Master_Receive(s_addr, (uint8_t *__near)buffer, rx_num);

        if( status == 0)                /* check start successfully     */
        {

/*--------------------------------------
    wait for receive end
--------------------------------------*/

            do
            {
                status = R_IIC_check_comstate();

            }while( IIC_USING == ( status & IIC_STS_MASK ) );
        }
        else
        {
            /* do nothing */
        }

    }
    else
    {
        /* do nothing */
    }

    if( NO_SLAVE != status )
    {
        R_IIC_StopCondition();          /* issue stop condition     */
    }
    else
    {
            /* do nothing */
    }

    return (status);

}
/***********************************************************************************************************************

    Long range timer functions

***********************************************************************************************************************/

/*******************************************************************************
* Function Name: wait_5ms
* Description  : This function wait for 5ms * times
* Arguments    : times of 5ms
* Return Value : none
********************************************************************************/
void wait_5ms(uint8_t times)
{
    g_005s_count = times ;                        /* set wait time (5ms unit)   */
    g_005s_status = FALSE;

/*--------------------------------------
   wait for remaining time
--------------------------------------*/

    while ( FALSE == g_005s_status )            /* wait for IT interrupt    */
    {
        HALT();
        NOP();
    }


}


/********************************************************************************
* Function Name: r_it_interrupt
* Description  : This function is 0.05sec interrupt service routine.
* Arguments    : None
* Return Value : None
*********************************************************************************/
static void r_it_interrupt(void)
{

    if ( FALSE == g_005s_status)
    {
        g_005s_count--;
        if( 0 == g_005s_count )
        {
            g_005s_status = TRUTH;      /* set 0.05s flag           */
        }
    }


}

/* End user code. Do not edit comment generated here */
